#! /bin/bash 

# nytrolin_set_affinity.sh : setup smp affinity
#version    : 1.3
#date       : 6 Dec February 2014

## Supported only on RHEL, CentOS, Oracle Linux, Ubuntu and Debian


if ([ -f /usr/bin/lsb_release ]); then
	distro=`lsb_release -si`
	case "$distro" in
		Redhat|RedHat*) ;;
		CentOS) ;;
		OracleServer) ;;
		Ubuntu) ;;
		Debian) ;;
		*)      echo "Exiting script...Only Redhat, CentOS, OracleServer, Ubuntu and Debian are supported"
		exit 1;
		;;
	esac

	if ([ -f /etc/init.d/irqbalance ]); then
		service irqbalance stop > /dev/null 2>&1
	fi;
else
	echo "lsb_release is not installed. Please install it and try running the script again"
	exit 2;
fi;

if [ `which awk | grep -i "no awk"`  ] ; then
	echo "awk is required, exiting..."
	exit 3;
fi;

# this is count of the number cpu's we have set the affinity for
cpu_affinity_count=0

# This is total number of cores
cpu_count=0
for i in /sys/devices/system/cpu/cpu[0-9]* ; do
	let cpu_count=${cpu_count}+1
done

[ ${cpu_count} -eq 0 ] && echo "CPU count is zero?" && exit

if [ $cpu_count -gt 64 ]; then
	echo "IRQ balancing will be done only on CPU 0 - 63"
	cpu_count=64
fi

scsi_host="/sys/class/scsi_host"
cd ${scsi_host}
subfolders=`ls -1`

msix_count=`cat /proc/interrupts | grep -cw nytrolin`
[ ${msix_count} -eq 0 ] && echo "No IRQs assigned to nytrolin" && exit 0

for i in ${subfolders};  do
	cd ${scsi_host}/${i};
	if [ `cat proc_name` != "nytrolin" ]; then
		continue;
	fi;
	index=0
	affinity=1
	cpu_affinity_count=0
	if [ ${cpu_count} -gt ${msix_count} ]; then
		let grouping=${cpu_count}/${msix_count}
		let grouping_mod=${cpu_count}%${msix_count}
		if [ ${grouping} -lt 2 ]; then
			cpu_grouping=2;
		elif [ ${grouping} -eq 2 ] && [ ${grouping_mod} -eq 0 ]; then
			cpu_grouping=2;
		elif [ ${grouping} -lt 4 ]; then
			cpu_grouping=4;
		elif [ ${grouping} -eq 4 ] && [ ${grouping_mod} -eq 0 ]; then
			cpu_grouping=4;
		elif [ ${grouping} -lt 8 ]; then
			cpu_grouping=8;
		elif [ ${grouping} -eq 8 ] && [ ${grouping_mod} -eq 0 ]; then
			cpu_grouping=8;
		else
			cpu_grouping=16;
		fi;
		msix_grouping=0;
	elif [ ${cpu_count} -eq ${msix_count} ] ; then
		cpu_grouping=0;
		msix_grouping=0;
	else
		msix_grouping=1;
		cpu_grouping=0;
	fi;

	echo "cpu grouping is $cpu_grouping"
	echo "msix grouping is $msix_grouping"
	echo -e "\n"
	echo number of core processors = ${cpu_count}
	echo msix vector count = ${msix_count}
	if [ ${cpu_grouping} -eq 0 ] ; then
		echo number of cores per msix vector = 1
	elif [ ${msix_grouping} -eq 0 ] ; then
		echo number of cores per msix vector = ${cpu_grouping}
	fi;
	echo -e "\n"
	a=`cat /proc/interrupts | egrep -w nytrolin |awk '{print $1;}'|cut -d':' -f 1`
	array=($a)
	iter=0;

	let affinity_low_32=0
	let affinity_high_32=0
	let affinity_value=0

	while [ ${msix_count} -gt ${index} ] ; do
		b=0
		irq_number=${array[$iter]}
		let iter=${iter}+1
		cd /proc/irq/${irq_number}
		if [ ${cpu_grouping} -ne 0 ]; then
			loop=1
			calculate_affinity=${affinity}
			let cpu_affinity_count=${cpu_affinity_count}+1

			while [ ${loop} -lt ${cpu_grouping} ]; do
				let affinity=${affinity}*2
				let calculate_affinity=${calculate_affinity}+${affinity}
				let loop=${loop}+1
				let cpu_affinity_count=${cpu_affinity_count}+1
			done;
			affinity_value=${calculate_affinity}
		else
			let cpu_affinity_count=${cpu_affinity_count}+1
			affinity_value=${affinity}
		fi

		# for CPU32..63, the mask should be set as 
		# echo "high_32,low_32" > smp_affinity

		if [ ${cpu_count} -gt 32 ]; then
			affinity_low_32=$(( ${affinity_value} & 0xFFFFFFFF ))
			affinity_high_32=$(( ${affinity_value} >> 32))
			affinity_high_32=$(( $affinity_high_32 & 0xFFFFFFFF ))
			b=`printf "%x,%x" ${affinity_high_32} ${affinity_low_32}`
		else 
			b=`printf "%x" ${affinity_value}`
		fi;

		if [ ${cpu_affinity_count} -lt ${cpu_count} ] || \
			[ ${cpu_affinity_count} -eq ${cpu_count} ]; then
			echo ${b} > smp_affinity
			echo -e "\tmsix index = ${index}, irq number =  ${irq_number}, cpu affinity mask = `cat smp_affinity`, cpu_affinity_count $cpu_affinity_count"
			let index=${index}+1
			let affinity=${affinity}*2
		else
			if [ ${msix_grouping} -eq 0 ] ; then
				break;
			else
				affinity=1
				cpu_affinity_count=1
				b=`printf "%x" ${affinity}`
				echo ${b} > smp_affinity
				echo -e "\tmsix index = ${index}, irq number =  ${irq_number}, cpu affinity mask = `cat smp_affinity`, cpu_affinity_count $cpu_affinity_count"
				let index=${index}+1
				let affinity=${affinity}*2
			fi;
		fi;
	done
	break;
done
